package org.processmining.miminer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.log.utils.XUtils;
import org.processmining.miminer.models.StateEntry;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.SetMultimap;

public class MIMinerConfiguration {

	public final XLog log;
	public XEventClassifier classifier;
	
	public SortedSet<String> traceAttributes = new TreeSet<>();
	public SortedSet<String> eventAttributes = new TreeSet<>();
	public SetMultimap<XEventClassifier, String> classifierStatesMap = HashMultimap.create();
	
	public SortedSet<String> types = new TreeSet<>();
	public SetMultimap<String, String> stateTypesMap = HashMultimap.create();
	public Map<String, String> typeSourceMap = new HashMap<>();
	public Map<String, String> typeAttributeMap = new HashMap<>();
	
	public List<String> instances = new ArrayList<>();
	public List<String> instanceTypeMap = new ArrayList<>(); // List interned Strings
	
	public List<List<StateEntry>> instanceStatesMap = new ArrayList<>();
	public List<Set<Integer>> instanceRelationsMap = new ArrayList<>(); // All SE from these instances are relevant
	public List<List<StateEntry>> instanceRelatedStatesMap = new ArrayList<>();
	public List<Integer> instanceParentMap = new ArrayList<>(); // Unused for now
	
	public MIMinerConfiguration(XLog log) {
		this.log = log;
		this.classifier = XUtils.getDefaultClassifier(log);
		
		// Initialise log data
		for (XTrace trace : log) {
			for (String attribute : trace.getAttributes().keySet()) {
				traceAttributes.add(attribute);
			}
			
			for (XEvent event : trace) {
				for (String attribute : event.getAttributes().keySet()) {
					eventAttributes.add(attribute);
				}
				
				for (XEventClassifier classifier : log.getClassifiers()) {
					classifierStatesMap.put(classifier, classifier.getClassIdentity(event));
				}
			}
		}
	}
}
