package org.processmining.miminer.configuration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JComponent;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.util.ui.wizard.ProMWizardStep;
import org.processmining.miminer.MIMinerConfiguration;
import org.processmining.miminer.models.StateEntry;
import org.processmining.miminer.models.StateEntry.RelationType;

import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;

public class ArtifactIdentifierMapping implements ProMWizardStep<MIMinerConfiguration> {
	
	public static final String traceSource = "trace";
	public static final String eventSource = "event";
	public static final boolean relateAllInstancesInTrace = true;
	
	private Table<String,String,Integer> typeInstanceNumberMap = HashBasedTable.create();

	public MIMinerConfiguration apply(MIMinerConfiguration configuration, JComponent configurationPanel) {
		ArtifactIdentifierMappingPanel artifactIdentifierMappingPanel = (ArtifactIdentifierMappingPanel) configurationPanel;
		
		for (String type : artifactIdentifierMappingPanel.typeSourceMap.keySet()) {
			configuration.typeSourceMap.put(type, (String) artifactIdentifierMappingPanel.typeSourceMap.get(type).getSelectedItem());
			configuration.typeAttributeMap.put(type, (String) artifactIdentifierMappingPanel.typeAttributeMap.get(type).getSelectedItem());
		}
		
		System.out.println(configuration.typeSourceMap);
		System.out.println(configuration.typeAttributeMap);
		
		// Parse the entire log and assign states to instances & create instance relations
		typeInstanceNumberMap.clear();
		configuration.instances.clear();
		configuration.instanceTypeMap.clear();
		configuration.instanceStatesMap.clear();
		configuration.instanceRelationsMap.clear();
		configuration.instanceRelatedStatesMap.clear();
		for (XTrace trace : configuration.log) {
			Set<Integer> traceInstances = new HashSet<>();
			
			for (XEvent event : trace) {
				for (String type : configuration.stateTypesMap.get(configuration.classifier.getClassIdentity(event))) {
					XAttribute instance = getInstanceAttribute(configuration, trace, event, type);
					if (instance != null) {
						Integer instanceNumber = getInstanceNumber(configuration, type, instance);
						Map<Integer, RelationType> relatedInstances = new HashMap<>();
						StateEntry entry = new StateEntry(instanceNumber, event, relatedInstances);
						configuration.instanceStatesMap.get(instanceNumber).add(entry);
						traceInstances.add(instanceNumber);
						
						for (String secondaryType : configuration.types) {
							if (secondaryType.equals(type)) continue;
							XAttribute secondaryInstance = getInstanceAttribute(configuration, trace, event, secondaryType);
							if (secondaryInstance != null) {
								Integer secondaryInstanceNumber = getInstanceNumber(configuration, secondaryType, secondaryInstance);
								
								if (traceSource.equals(configuration.typeSourceMap.get(secondaryType))) {
									// Trace attributes are contained in every event, so all SE of such an AI are relevant to any given AI
									configuration.instanceRelationsMap.get(instanceNumber).add(secondaryInstanceNumber);
								}
//								else if (eventSource.equals(configuration.typeSourceMap.get(secondaryType))) {
//									// If we have a reference to an AI then we want to know its possible state, so all SE are relevant
//									configuration.instanceRelationsMap.get(instanceNumber).add(secondaryInstanceNumber);
//								}
								
								// If a SE contains a reference to another AI then it is relevant for that AI
								configuration.instanceRelatedStatesMap.get(secondaryInstanceNumber).add(entry);
								relatedInstances.put(secondaryInstanceNumber, RelationType.UPDATE);
							}
						}
					}
					else {
						// SE with a missing AI
					}
				}
			}
			
			if (relateAllInstancesInTrace) {
				for (Integer instance : traceInstances) {
					configuration.instanceRelationsMap.get(instance).addAll(traceInstances);
				}
			}
		}
		
		System.out.println(configuration.instances);
		System.out.println(configuration.instanceTypeMap);
		System.out.println(configuration.instanceStatesMap.get(0));
		System.out.println(configuration.instanceRelationsMap.get(0));
		System.out.println(configuration.instanceRelatedStatesMap.get(0));
		
		return configuration;
	}
	
	private XAttribute getInstanceAttribute(MIMinerConfiguration configuration, XTrace trace, XEvent event, String type) {
		XAttribute instance = null;
		String source = configuration.typeSourceMap.get(type);
		String attribute = configuration.typeAttributeMap.get(type);
		
		if (traceSource.equals(source)) {
			instance = trace.getAttributes().get(attribute);
		}
		else if (eventSource.equals(source)) {
			instance = event.getAttributes().get(attribute);
		}
		
		return instance;
	}
	
	private Integer getInstanceNumber(MIMinerConfiguration configuration, String type, XAttribute instance) {
		String instanceName = instance.toString();
		Integer instanceNumber = typeInstanceNumberMap.get(type, instanceName);
		if (instanceNumber == null) {
			instanceNumber = configuration.instances.size();
			typeInstanceNumberMap.put(type, instanceName, instanceNumber);
			configuration.instances.add(instanceName);
			configuration.instanceTypeMap.add(type.intern());
			configuration.instanceStatesMap.add(new ArrayList<StateEntry>());
			configuration.instanceRelationsMap.add(new HashSet<Integer>());
			configuration.instanceRelatedStatesMap.add(new ArrayList<StateEntry>());
		}
		
		return instanceNumber;
	}

	public boolean canApply(MIMinerConfiguration configuration, JComponent configurationPanel) {
		return true;
	}

	public JComponent getComponent(MIMinerConfiguration configuration) {
		return new ArtifactIdentifierMappingPanel(configuration);
	}

	public String getTitle() {
		return "Multi Instance Miner Configuration";
	}

}
