package org.processmining.miminer.configuration;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.miminer.MIMinerConfiguration;

public class ArtifactIdentifierMappingPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public Map<String, ProMComboBox<String>> typeSourceMap = new HashMap<>();
	public Map<String, ProMComboBox<String>> typeAttributeMap = new HashMap<>();
	
	public ArtifactIdentifierMappingPanel(MIMinerConfiguration configuration) {
		super();
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		
		System.out.println(configuration.types);
		System.out.println(configuration.stateTypesMap);
		
		JLabel attributeLabel = new JLabel("For every artifact, select the attribute representing its artifact instance IDs:",
				SwingConstants.CENTER);
		attributeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		attributeLabel.setFont(new Font("Dialog", Font.BOLD, 14));
		add(attributeLabel);
		
		add(Box.createRigidArea(new Dimension(0,10)));
		
		for (String type : configuration.types) {
			JPanel attributePanel = new JPanel();
			attributePanel.setLayout(new BoxLayout(attributePanel, BoxLayout.LINE_AXIS));
			attributePanel.setBackground(new Color(150,150,150));
			add(attributePanel);
			
			JLabel typeLabel = new JLabel(type + ":", SwingConstants.CENTER);
			attributePanel.add(typeLabel);
			
			attributePanel.add(Box.createHorizontalGlue());
			
			ProMComboBox<String> sourceSelection = new ProMComboBox<>(new DefaultComboBoxModel<String>(
					new String[] { ArtifactIdentifierMapping.traceSource, ArtifactIdentifierMapping.eventSource } ));
			sourceSelection.setPreferredSize(new Dimension(100, 30));
			sourceSelection.addItemListener(new SourceListener(configuration, type));
			attributePanel.add(sourceSelection);
			typeSourceMap.put(type, sourceSelection);
			
			ProMComboBox<String> attributeSelection = new ProMComboBox<>(new DefaultComboBoxModel<String>(
					configuration.traceAttributes.toArray(new String[0])));
			attributeSelection.setPreferredSize(new Dimension(300, 30));
			attributePanel.add(attributeSelection);
			typeAttributeMap.put(type, attributeSelection);
		}
	}
	
	private class SourceListener implements ItemListener {

		MIMinerConfiguration configuration;
		String type;
		
		public SourceListener(MIMinerConfiguration configuration, String type) {
			this.configuration = configuration;
			this.type = type;
		}

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>) typeAttributeMap.get(type).getModel();
				model.removeAllElements();
				
				if (ArtifactIdentifierMapping.traceSource.equals(e.getItem())) {
					for (String attribute : configuration.traceAttributes) {
						model.addElement(attribute);
					}
				}
				else if (ArtifactIdentifierMapping.eventSource.equals(e.getItem())) {
					for (String attribute : configuration.eventAttributes) {
						model.addElement(attribute);
					}
				}
			}
		}
		
	}
}
