package org.processmining.miminer.configuration;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.deckfour.xes.classification.XEventClassifier;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.miminer.MIMinerConfiguration;

public class StateArtifactMappingPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public ProMComboBox<XEventClassifier> classifierSelection;
	public ProMTable table;
	public DefaultTableModel tableModel;
	public Map<XEventClassifier, Map<String, String>> classifierStateArtifactMap = new HashMap<>();

	public StateArtifactMappingPanel(MIMinerConfiguration configuration) {
		super();
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		
		for (XEventClassifier classifier : configuration.classifierStatesMap.keySet()) {
			classifierStateArtifactMap.put(classifier, new HashMap<String,String>());
			
			for (String state : configuration.classifierStatesMap.get(classifier)) {
				classifierStateArtifactMap.get(classifier).put(state, "");
			}
		}
		
		JLabel classifierLabel = new JLabel("Please select classifier to use:", SwingConstants.CENTER);
		classifierLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		classifierLabel.setFont(new Font("Dialog", Font.BOLD, 14));
		add(classifierLabel);
		
		classifierSelection = new ProMComboBox<>(new DefaultComboBoxModel<XEventClassifier>(
				configuration.log.getClassifiers().toArray(new XEventClassifier[0])));
		classifierSelection.setPreferredSize(new Dimension(750, 30));
		classifierSelection.addItemListener(new ClassifierListener());
		add(classifierSelection);
		
		JLabel tableLabel = new JLabel("<html>Assign each state to one or more artifacts (use ; to separate multiple artifact names). "
				+ "<br/>Use Ctrl-C and Ctrl-V to copy and paste the focussed table cell.</html>",
				SwingConstants.CENTER);
		tableLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		tableLabel.setFont(new Font("Dialog", Font.BOLD, 14));
		add(tableLabel);
		
		tableModel = new ArtifactTableModel();
		tableModel.addColumn("State Name");
		tableModel.addColumn("Artifact Name");
		
		updateTableContent();
		
		table = new ProMTable(tableModel);
		table.getTableHeader().setReorderingAllowed(false);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getTable().setTransferHandler(new CopyTableTransferHandler());
		table.setPreferredSize(new Dimension(750, 500));
		table.setRowSorter(new TableRowSorter<TableModel>(table.getTable().getModel()));
		((TableRowSorter)table.getRowSorter()).toggleSortOrder(0);
		
		add(table);
	}
	
	public void updateTableContent() {
		Map<String, String> stateArtifactMap = classifierStateArtifactMap.get(classifierSelection.getSelectedItem());
		
		for (String state : stateArtifactMap.keySet()) {
			tableModel.addRow(new Object[] { state, stateArtifactMap.get(state) } );
		}
	}
	
	public void updateStateNames(XEventClassifier classifier) {
		for (int i = 0; i < tableModel.getRowCount(); i++) {
			Map<String, String> stateArtifactMap = classifierStateArtifactMap.get(classifier);
			stateArtifactMap.put((String) tableModel.getValueAt(i, 0), (String) tableModel.getValueAt(i, 1));
		}
	}
	
	private class ClassifierListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				updateStateNames((XEventClassifier) e.getItem());
				tableModel.setRowCount(0);
			}
			
			if (e.getStateChange() == ItemEvent.SELECTED) {
				updateTableContent();
			}
		}
		
	}
	
	private class ArtifactTableModel extends DefaultTableModel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public boolean isCellEditable(int row, int column){
			if (column == 0) {
				return false;
			}
			else return true;
		}
		
		public Class getColumnClass(int column) {
			switch (column) {
				case 0: case 1:
					return String.class;
				default:
					return Object.class;
			}
		}
	}
}
