package org.processmining.miminer.models;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.processmining.miminer.visualisation.ArtifactTypeModelVisualisation;
import org.processmining.miminer.visualisation.VisualisationConnections;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.visualisation.DotPanel;

import com.google.common.collect.HashBasedTable;
import com.google.common.collect.HashMultiset;
import com.google.common.collect.Multiset;
import com.google.common.collect.Table;

public class ArtifactTypeModel {

	public String artifactType;
	public Multiset<ArtifactState> artifactStates = HashMultiset.create();
	public Multiset<ArtifactTransition> artifactTransitions = HashMultiset.create();
	public Map<ArtifactState, DescriptiveStatistics> statePeriods = new HashMap<>();
	
//	public Table<ArtifactTransition, ArtifactTransition, Integer> transitionPreTransitionCounts = HashBasedTable.create(); // tran x tran x nr
//	public Table<ArtifactTransition, Multiset<ArtifactTransition>, Integer> transitionPreMarkings = HashBasedTable.create(); // tran x total marking change x nr
	public Table<ArtifactTransition, ArtifactState, Integer> transitionPostStateCounts = HashBasedTable.create(); // tran x state x nr
	public Table<ArtifactTransition, ArtifactTransition, Integer> transitionPostTransitionCounts = HashBasedTable.create(); // tran x tran x nr
	public Table<ArtifactTransition, Multiset<ArtifactTransition>, Integer> transitionPostMultisets = HashBasedTable.create(); // tran x total transition change x nr
	public Table<ArtifactTransition, Multiset<ArtifactTransition>, Integer> transitionPostMarkings = HashBasedTable.create(); // tran x total marking change x nr
	
	public Table<ArtifactState, ArtifactTransition, Integer> relatedTransitionCounts = HashBasedTable.create(); // state x tran x nr
	public Table<ArtifactState, Multiset<ArtifactTransition>, Integer> relatedTransitionMultisets = HashBasedTable.create(); // state x tran x nr
	public Table<ArtifactState, Multiset<ArtifactTransition>, Integer> relatedTransitionMarkings = HashBasedTable.create(); // state x total marking x nr
	
	public Table<ArtifactState, ArtifactState, DescriptiveStatistics> relatedStatePeriods = HashBasedTable.create(); // state x state x dscr
	public Table<ArtifactState, Multiset<ArtifactState>, DescriptiveStatistics> relatedMarkings = HashBasedTable.create(); // state x total marking x dscr
	public Map<String,Table<ArtifactState, Multiset<ArtifactState>, DescriptiveStatistics>> relatedTypeMarkings = new HashMap<>();
//	public Table<ArtifactState, Multiset<ArtifactState>, DescriptiveStatistics> relatedTypeMarkings = HashBasedTable.create(); // state x type marking x dscr
//	public Table<ArtifactState, String, Multiset<ArtifactState>> relatedStateCounts = HashBasedTable.create(); //?? state x state x number
	
	public ArtifactTypeModel(String artifactType) {
		this.artifactType = artifactType;
	}
	
	public ArtifactState addState(String stateName) {
		ArtifactState state = new ArtifactState(artifactType, stateName);
		artifactStates.add(state);
		return state;
	}
	
	public ArtifactTransition addTransition(String fromState, String toState) {
		ArtifactTransition transition = new ArtifactTransition(artifactType, fromState, toState);
		artifactTransitions.add(transition);
		return transition;
	}
	
	public void addStatePeriod(String stateName, long period) {
		ArtifactState state = new ArtifactState(artifactType, stateName);
		DescriptiveStatistics statistics = statePeriods.get(state);
		if (statistics == null) {
			statistics = new DescriptiveStatistics();
			statePeriods.put(state, statistics);
		}
		statistics.addValue(period);
	}
	
	public ArtifactTransition addTransitionPostStateCount(ArtifactTransition transition, String artifactType, String stateName) {
		ArtifactTransition relatedTransition = new ArtifactTransition(artifactType, stateName, stateName);
		ArtifactState relatedState = new ArtifactState(artifactType, stateName);
		Integer count = transitionPostStateCounts.get(transition, relatedState);
		if (count == null) {
			count = 0;
		}
		transitionPostStateCounts.put(transition, relatedState, count+1);
		return relatedTransition;
	}
	
	public ArtifactTransition addTransitionPostTransitionCount(ArtifactTransition transition, String artifactType, String fromState,
			String toState) {
		ArtifactTransition relatedTransition = new ArtifactTransition(artifactType, fromState, toState);
		Integer count = transitionPostTransitionCounts.get(transition, relatedTransition);
		if (count == null) {
			count = 0;
		}
		transitionPostTransitionCounts.put(transition, relatedTransition, count+1);
		return relatedTransition;
	}
	
	public void addTransitionPostMultiset(ArtifactTransition transition, Multiset<ArtifactTransition> multiset) {
		Integer count = transitionPostMultisets.get(transition, multiset);
		if (count == null) {
			count = 0;
		}
		transitionPostMultisets.put(transition, multiset, count+1);
	}
	
	public void addTransitionPostMarking(ArtifactTransition transition, Multiset<ArtifactTransition> marking) {
		Integer count = transitionPostMarkings.get(transition, marking);
		if (count == null) {
			count = 0;
		}
		transitionPostMarkings.put(transition, marking, count+1);
	}
	
	public ArtifactTransition addRelatedTransitionCount(ArtifactState state, String artifactType, String fromState,
			String toState) {
		ArtifactTransition relatedTransition = new ArtifactTransition(artifactType, fromState, toState);
		Integer count = relatedTransitionCounts.get(state, relatedTransition);
		if (count == null) {
			count = 0;
		}
		relatedTransitionCounts.put(state, relatedTransition, count+1);
		return relatedTransition;
	}
	
	public void addRelatedTransitionMultisets(ArtifactState state, Multiset<ArtifactTransition> multiset) {
		Integer count = relatedTransitionMultisets.get(state, multiset);
		if (count == null) {
			count = 0;
		}
		relatedTransitionMultisets.put(state, multiset, count+1);
	}
	
	public void addRelatedTransitionMarking(ArtifactState state, Multiset<ArtifactTransition> marking) {
		Integer count = relatedTransitionMarkings.get(state, marking);
		if (count == null) {
			count = 0;
		}
		relatedTransitionMarkings.put(state, marking, count+1);
	}
	
	public ArtifactState addRelatedStatePeriod(ArtifactState state, String artifactType, String stateName, long period) {
		ArtifactState relatedState = new ArtifactState(artifactType, stateName);
		DescriptiveStatistics statistics = relatedStatePeriods.get(state, relatedState);
		if (statistics == null) {
			statistics = new DescriptiveStatistics();
			relatedStatePeriods.put(state, relatedState, statistics);
		}
		statistics.addValue(period);
		return relatedState;
	}
	
	public void addRelatedMarking(ArtifactState state, Multiset<ArtifactState> marking, long period) {
		DescriptiveStatistics statistics = relatedMarkings.get(state, marking);
		if (statistics == null) {
			statistics = new DescriptiveStatistics();
			relatedMarkings.put(state, marking, statistics);
		}
		statistics.addValue(period);
	}
	
	public void addRelatedTypeMarkings(String type, ArtifactState state, Multiset<ArtifactState> marking, long period) {
		Table<ArtifactState, Multiset<ArtifactState>, DescriptiveStatistics> typeMarking = relatedTypeMarkings.get(type);
		if (typeMarking == null) {
			typeMarking = HashBasedTable.create();
			relatedTypeMarkings.put(type, typeMarking);
		}
		
		DescriptiveStatistics statistics = typeMarking.get(state, marking);
		if (statistics == null) {
			statistics = new DescriptiveStatistics();
			typeMarking.put(state, marking, statistics);
		}
		statistics.addValue(period);
	}
	
	public DotPanel createGraph(VisualisationConnections connections) {
		ArtifactTypeModelVisualisation graph = new ArtifactTypeModelVisualisation(new Dot(), connections, this);
		connections.graphs.put(artifactType, graph);
		return graph;
	}
	
	public class ArtifactState {
		public String artifactType;
		public String stateName;
		
		public ArtifactState(String artifactType, String stateName) {
			this.artifactType = artifactType.intern();
			this.stateName = stateName.intern();
		}

		public String toString() {
			return artifactType + ":" + stateName;
		}

		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((artifactType == null) ? 0 : artifactType.hashCode());
			result = prime * result + ((stateName == null) ? 0 : stateName.hashCode());
			return result;
		}
		
		public boolean equals(Object obj) {
			if (this == obj) return true;
			if (obj == null) return false;
			if (getClass() != obj.getClass()) return false;
			ArtifactState other = (ArtifactState) obj;
			if (artifactType == null) {
				if (other.artifactType != null)
					return false;
			} else if (!artifactType.equals(other.artifactType))
				return false;
			if (stateName == null) {
				if (other.stateName != null)
					return false;
			} else if (!stateName.equals(other.stateName))
				return false;
			return true;
		}
	}
	
	public class ArtifactTransition {
		public String artifactType;
		public String fromState;
		public String toState;
		
		public ArtifactTransition(String artifactType, String fromState, String toState) {
			this.artifactType = artifactType.intern();
			this.fromState = fromState.intern();
			this.toState = toState.intern();
		}
		
		public String toString() {
			if (fromState.equals(toState)) {
				return artifactType + ":(" + fromState + ")";
			}
			else {
				return artifactType + ":(" + fromState + "," + toState + ")";
			}
		}

		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((artifactType == null) ? 0 : artifactType.hashCode());
			result = prime * result + ((fromState == null) ? 0 : fromState.hashCode());
			result = prime * result + ((toState == null) ? 0 : toState.hashCode());
			return result;
		}
		
		public boolean equals(Object obj) {
			if (this == obj) return true;
			if (obj == null) return false;
			if (getClass() != obj.getClass()) return false;
			ArtifactTransition other = (ArtifactTransition) obj;
			if (artifactType == null) {
				if (other.artifactType != null)
					return false;
			} else if (!artifactType.equals(other.artifactType))
				return false;
			if (fromState == null) {
				if (other.fromState != null)
					return false;
			} else if (!fromState.equals(other.fromState))
				return false;
			if (toState == null) {
				if (other.toState != null)
					return false;
			} else if (!toState.equals(other.toState))
				return false;
			return true;
		}
	}

	public ArtifactTransition transitionFromState(ArtifactState state) {
		return new ArtifactTransition(state.artifactType, state.stateName, state.stateName);
	}

	public ArtifactState getState(String artifactType, String stateName) {
		return new ArtifactState(artifactType, stateName);
	}
	
	public ArtifactTransition getTransition(String artifactType, String fromState, String toState) {
		return new ArtifactTransition(artifactType, fromState, toState);
	}
}
