package org.processmining.miminer.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.wizard.ListWizard;
import org.processmining.framework.util.ui.wizard.ProMWizardDisplay;
import org.processmining.miminer.MIMinerConfiguration;
import org.processmining.miminer.configuration.ArtifactIdentifierMapping;
import org.processmining.miminer.configuration.StateArtifactMapping;

@Plugin(
		name = "Multi Instance Miner Configuration",
		level = PluginLevel.Regular,
		categories = { PluginCategory.Filtering },
		parameterLabels = { "Event Log" },
		returnLabels = { "Configuration" },
		returnTypes = { MIMinerConfiguration.class },
		userAccessible = true,
		help = "Creates a configuration as input for the Multi Instance Miner, as described in: "
				+ "\"To Be Written\" "
				+ "by M.L. van Eck, N. Sidorova, W.M.P. van der Aalst."
				+ "<br/><font color=\"rgb(120,120,120)\">qq</font><br/>"
				+ "Effect: TBD",
		mostSignificantResult = 1)
public class MIMinerConfigurationPlugin {
	@UITopiaVariant(
			uiLabel = "MI Miner Configuration",
			affiliation = "Eindhoven University of Technology",
			author = "Maikel L. van Eck",
			email = "m.l.v.eck@tue.nl",
			pack = "MIMiner")
	@PluginVariant(variantLabel = "Create a Configuration for the MI Miner", requiredParameterLabels = { 0 })
	public MIMinerConfiguration createConfiguration(final UIPluginContext context, XLog log) {
		MIMinerConfiguration configuration = new MIMinerConfiguration(log);
		ListWizard<MIMinerConfiguration> wizard = new ListWizard<MIMinerConfiguration>(
				new StateArtifactMapping(),
				new ArtifactIdentifierMapping());
		configuration = ProMWizardDisplay.show(context, wizard, configuration);
		if (configuration == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		return configuration;
	}
}
