package org.processmining.miminer.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.miminer.MIMiner;
import org.processmining.miminer.MIMinerConfiguration;
import org.processmining.miminer.MIMinerResults;

@Plugin(
		name = "Multi Instance Miner",
		level = PluginLevel.Regular,
		categories = { PluginCategory.Discovery, PluginCategory.Enhancement },
		parameterLabels = { "Artifact Log", "Configuration" },
		returnLabels = { "Artifact Models & Interactions", "Artifact Log" },
		returnTypes = { MIMinerResults.class, XLog.class },
		userAccessible = true,
		help = "The Multi Instance Miner, as described in: "
				+ "\"To Be Written\" "
				+ "by M.L. van Eck, N. Sidorova, W.M.P. van der Aalst.<br/>"
				+ "Discovers a model for each artifact type in the log."
				+ " The interactive visualisation of the models shows behavioural relations between artifacts and various statistics."
				+ "<br/><font color=\"rgb(120,120,120)\">qq</font><br/>"
				+ "Input assumptions:<br/>"
				+ "TBD",
		mostSignificantResult = 1)
public class MIMinerPlugin {
	@UITopiaVariant(
			uiLabel = "MI Miner",
			affiliation = "Eindhoven University of Technology",
			author = "Maikel L. van Eck",
			email = "m.l.v.eck@tue.nl",
			pack = "MIMiner")
	@PluginVariant(variantLabel = "Mine Multi Instance Artifact Models", requiredParameterLabels = { 0, 1 })
	public Object[] mineCompositeStateMachine(final PluginContext context, XLog log, MIMinerConfiguration configuration) {
		if (!log.equals(configuration.log)) {
			return cancelPlugin(context, "Log and configuration don't match.");
		}
		
		return MIMiner.mine(context, configuration);
	}
	
	@UITopiaVariant(
			uiLabel = "MI Miner",
			affiliation = "Eindhoven University of Technology",
			author = "Maikel L. van Eck",
			email = "m.l.v.eck@tue.nl",
			pack = "MIMiner")
	@PluginVariant(variantLabel = "Mine Multi Instance Artifact Models", requiredParameterLabels = { 0 })
	public Object[] mineCompositeStateMachine(final UIPluginContext context, XLog log) {
		MIMinerConfiguration configuration = null;
		
		try {
			configuration = context.tryToFindOrConstructFirstNamedObject(MIMinerConfiguration.class, 
					"Multi Instance Miner Configuration", null, "", log);
		} catch (Exception e) {
			cancelPlugin(context, "No configuration created.");
		}
		
		return MIMiner.mine(context, configuration);
	}
	
	public Object[] cancelPlugin(PluginContext context, String errorMessage) {
		System.out.println(errorMessage);
		context.log(errorMessage);
		context.getFutureResult(0).cancel(true);
		context.getFutureResult(1).cancel(true);
		return null;
	}
}
