package org.processmining.miminer.visualisation;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayer;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumnModel;

import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTableWithoutPanel;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.miminer.MIMinerResults;

public class MIMinerInteractiveVisualisation extends ProMSplitPane {
	
	private static final long serialVersionUID = -7020146564517851637L;
	
	final int numberProcessesShown = 3;
	final GraphicsDevice gd = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
	final int processWidth = gd.getDisplayMode().getWidth() / numberProcessesShown;
	
	final static String artifactRelationModel = "Artifact Relations";
	final static String artifactTypeModel = "Artifact Lifecycles";
	
	public MIMinerResults results;
	public VisualisationConnections connections;

	public MIMinerInteractiveVisualisation(MIMinerResults results) {
		super(ProMSplitPane.VERTICAL_SPLIT);
		
		this.results = results;
		connections = new VisualisationConnections();
		
		JLayer<JComponent> modelLayer = createModelLayer();
//		ProMTableWithoutPanel statisticsPane = createStatisticsPane();
//		MetricsPanel metricsPane = new MetricsPanel(results, connections, processWidth * numberProcessesShown);
		RelationsPanel metricsPane = new RelationsPanel(results, connections, processWidth * numberProcessesShown);
		
		// Create graph and statistics wrapper
		JSplitPane graphStatisticsWrapper = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		graphStatisticsWrapper.setBackground(Color.LIGHT_GRAY);
		graphStatisticsWrapper.setTopComponent(modelLayer);
//		graphStatisticsWrapper.setBottomComponent(statisticsPane);
		graphStatisticsWrapper.setResizeWeight(1.0);
		graphStatisticsWrapper.setDividerSize(0);
		graphStatisticsWrapper.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		connections.graphStatisticsWrapper = graphStatisticsWrapper;
		// TODO: check if reference is needed: only for switching between statistics and interaction
		
		setTopComponent(graphStatisticsWrapper);
		setBottomComponent(metricsPane);
		setResizeWeight(1.0);
		setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		connections.topComponent = this;
	}
	
	private JPanel createGraphPanel(String type) {
		JLabel label = new JLabel(type, SwingConstants.CENTER);
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		label.setFont(new Font("Dialog", Font.BOLD, 14));
		
		JPanel graphPanel = new JPanel();
		graphPanel.setLayout(new BoxLayout(graphPanel, BoxLayout.PAGE_AXIS));
		graphPanel.add(label);
		graphPanel.add(results.artifactTypeModels.get(type).createGraph(connections));
		graphPanel.setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, Color.black));
		graphPanel.setPreferredSize(new Dimension(processWidth,0));
		graphPanel.setBackground(Color.lightGray);
		connections.graphWrappers.put(type, graphPanel);
		
		graphPanel.setVisible(true);
		
		return graphPanel;
	}
	
	private JLayer<JComponent> createModelLayer() {
		JPanel graphsWrapper = new JPanel();
		graphsWrapper.setLayout(new BoxLayout(graphsWrapper, BoxLayout.LINE_AXIS));
		graphsWrapper.setBackground(Color.LIGHT_GRAY);
		
		// Create a graph for each artifact type model
		for (String type : results.configuration.types) {
			graphsWrapper.add(createGraphPanel(type));
		}
		
		// Create a graph with artifact relations
		JPanel artifactRelationPanel = new JPanel();
		artifactRelationPanel.setLayout(new BoxLayout(artifactRelationPanel, BoxLayout.PAGE_AXIS));
		artifactRelationPanel.setPreferredSize(new Dimension(3*processWidth - 200,0));
		artifactRelationPanel.add(results.artifactRelationModel.createGraph());
		artifactRelationPanel.setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, Color.black));
		artifactRelationPanel.setBackground(Color.lightGray);
		artifactRelationPanel.setVisible(false);
		graphsWrapper.add(artifactRelationPanel);
		connections.artifactRelationPanel = artifactRelationPanel;
		
		// Embed graphs in a horizontal scroll pane
		ProMScrollPane scrollPane = new ProMScrollPane(graphsWrapper); 
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		// Create config panel
		GridBagLayout configLayout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel configPanel = new JPanel(configLayout);
		configPanel.setBackground(Color.LIGHT_GRAY);
		
		// Create model switch
		ProMComboBox<String> modelSwitch = new ProMComboBox<>(new DefaultComboBoxModel<String>(
				new String[] { artifactTypeModel, artifactRelationModel } ));
		modelSwitch.addItemListener(new ModelSwitchListener());
		
		modelSwitch.setMinimumSize(new Dimension(160, 30));
		configPanel.add(modelSwitch, gbc);
		
		// Create list of models
		DefaultListModel<String> modelNames = new DefaultListModel<>();
		for (String type : results.configuration.types) {
			modelNames.addElement(type);
		}
		ProMList<String> modelList = new ProMList<>("Model List", modelNames);
		modelList.getList().setSelectionInterval(0, results.configuration.types.size());
		modelList.addListSelectionListener(new ModelListSelectionListener(modelNames, modelSwitch));
		modelList.setMinimumSize(new Dimension(100, 100));
		((ProMScrollPane) modelList.getComponent(2)).setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		gbc.gridy = 1;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1;
		gbc.weighty = 1;
		configPanel.add(modelList, gbc);
		
		// Put together the config panel and the scroll pane
		JSplitPane modelPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		modelPane.setBackground(Color.LIGHT_GRAY);
		modelPane.setTopComponent(configPanel);
		modelPane.setBottomComponent(scrollPane);
		modelPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		modelPane.setUI(new BasicSplitPaneUI() {
			@Override
			public BasicSplitPaneDivider createDefaultDivider() {
				final BasicSplitPaneDivider divider = new ImprovedSplitPaneDivider(this);
				divider.setBackground(WidgetColors.COLOR_ENCLOSURE_BG);
				divider.setForeground(Color.LIGHT_GRAY);
				return divider;
			}
		});
		
		ProgressUI progressUI = new ProgressUI();
		connections.progressUI = progressUI;
		return new JLayer<>(modelPane, progressUI);
	}
	
	private ProMTableWithoutPanel createStatisticsPane() {
		// Create table model
		DefaultTableModel tableModel = new DefaultTableModel();
		
		tableModel.addColumn("Name");
		tableModel.addColumn("Count");
		tableModel.addColumn("Instances");
		tableModel.addColumn("Average Per Instance");
		tableModel.addColumn("Support (Count)");
		tableModel.addColumn("Median Sojourn Time");
		tableModel.addColumn("Average Sojourn Time Per Instance");
		tableModel.addColumn("Support (Time)");
		
		tableModel.addRow(new Object[] {"Name", "Count", "Instances", "Average Per Instance", "Support (Count)", "Median Sojourn Time",
				"Average Sojourn Time Per Instance", "Support (Time)"});
		tableModel.addRow(new Object[] {""});
		
		ProMTableWithoutPanel statisticsPane = new ProMTableWithoutPanel(tableModel);
		statisticsPane.setBorder(BorderFactory.createMatteBorder(0, 0, 0, 1, Color.LIGHT_GRAY));
		connections.statisticsPane = statisticsPane;
		
		// Set table appearance
		statisticsPane.setMinimumSize(new Dimension(processWidth * numberProcessesShown,0));
		statisticsPane.setPreferredSize(new Dimension(processWidth * numberProcessesShown,30));
		TableColumnModel columnModel = statisticsPane.getColumnModel();
		columnModel.getColumn(0).setPreferredWidth(2*(processWidth * numberProcessesShown) / (columnModel.getColumnCount()+1));
		
		for (int i = 1; i < columnModel.getColumnCount(); i++) {
			columnModel.getColumn(i).setPreferredWidth((processWidth * numberProcessesShown) / (columnModel.getColumnCount()+1));
		}
		
		return statisticsPane;
	}
	
	private class ModelSwitchListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (e.getItem() == artifactRelationModel) {
					connections.artifactRelationPanel.setVisible(true);
					for (String model : connections.graphWrappers.keySet()) {
						connections.graphWrappers.get(model).setVisible(false);
					}
				}
				else if (e.getItem() == artifactTypeModel) {
					connections.artifactRelationPanel.setVisible(false);
					for (String model : connections.graphWrappers.keySet()) {
						connections.graphWrappers.get(model).setVisible(connections.visibleModels.get(model));
					}
				}
			}
		}
		
	}
	
	private class ModelListSelectionListener implements ListSelectionListener {

		private DefaultListModel<String> listModel;
		private ProMComboBox<String> modelSwitch;
		
		public ModelListSelectionListener(DefaultListModel<String> modelNames, ProMComboBox<String> modelSwitch) {
			listModel = modelNames;
			this.modelSwitch = modelSwitch;
			for (int i = 0; i < modelNames.size(); i++) {
				connections.visibleModels.put(modelNames.getElementAt(i), true);
			}
		}
		
		public void valueChanged(ListSelectionEvent e) {
			JList list = (JList) e.getSource();
			for (int i = 0; i < listModel.size(); i++) {
				connections.visibleModels.put(listModel.getElementAt(i), list.isSelectedIndex(i));
				if (modelSwitch.getSelectedItem() == artifactTypeModel) {
					connections.graphWrappers.get(listModel.getElementAt(i)).setVisible(list.isSelectedIndex(i));
				}
			}
//			connections.topComponent.validate();
//			connections.metricsPane.table.clearSelection();
//			connections.graphStatisticsWrapper.setBottomComponent(connections.statisticsPane);
//			connections.statisticsPane.repaint();
			// TODO: switch between interaction explanation and selection statistics
		}
		
	}
}
