package org.processmining.miminer.visualisation;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Enumeration;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.processmining.framework.util.ui.widgets.ProMTable;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.miminer.MIMinerResults;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class MetricsPanel extends JSplitPane {

	private static final long serialVersionUID = 3468816272629744389L;
	
	public JPanel panel;
	public ProMTable table;
	
	private TableRowSorter<TableModel> sorter;
	
	private Dimension singleSize, doubleSize, tripleSize;
	private String[] relationTypes = { "State co-occurrences", "State co-occurrences (Hide symmetric half)", "Transition co-occurrences",
			"Forward-looking co-occurrences" };
	private JComboBox switchRelationTypeBox = SlickerFactory.instance().createComboBox(relationTypes);
	private JCheckBox toggleTypeNameBox = SlickerFactory.instance().createCheckBox("Show type names", false);
	private ProMTextField nameFilter = new ProMTextField("Filter by name");
	private ProMTextField minSupport = new ProMTextField("Minimum Support");
	private ProMTextField minConfidence = new ProMTextField("Minimum Confidence");
	private ProMTextField minLift = new ProMTextField("Minimum Lift");
	private ProMTextField minConviction = new ProMTextField("Minimum Conviction");
	private ProMTextField minCosine = new ProMTextField("Minimum Cosine");
	private ProMTextField minJaccard = new ProMTextField("Minimum Jaccard");
	private ProMTextField minPhi = new ProMTextField("Minimum Phi");
	
	public MetricsPanel(MIMinerResults results, VisualisationConnections connections, int width) {
		super(JSplitPane.VERTICAL_SPLIT);
		
		this.table = new ProMTable(metricsTableModel);
		this.panel = new JPanel(new GridBagLayout());
		
		setBackground(Color.LIGHT_GRAY);
		setBottomComponent(table);
		setTopComponent(panel);
		setResizeWeight(0.0);
		setDividerSize(0);
		setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
		
		createTable(width);
		
		sorter = new TableRowSorter<TableModel>(table.getTable().getModel());
		table.setRowSorter(sorter);
		
		// TODO: add content
//		addRelations();
//		sorter.setRowFilter(createCombinedFilter());
		
		// TODO: add filtering listeners
//		switchRelationTypeBox.addActionListener(new SwitchRelationTypeActionListener());
//		toggleTypeNameBox.addActionListener(new ToggleProcessNameActionListener());
//		nameFilter.getDocument().addDocumentListener(new NameFilterDocumentListener());
//		minSupport.getDocument().addDocumentListener(new MinMetricDocumentListener(minSupport, 2));
//		minConfidence.getDocument().addDocumentListener(new MinMetricDocumentListener(minConfidence, 3));
//		minLift.getDocument().addDocumentListener(new MinMetricDocumentListener(minLift, 4));
//		minConviction.getDocument().addDocumentListener(new MinMetricDocumentListener(minConviction, 5));
//		minCosine.getDocument().addDocumentListener(new MinMetricDocumentListener(minCosine, 6));
//		minJaccard.getDocument().addDocumentListener(new MinMetricDocumentListener(minJaccard, 7));
//		minPhi.getDocument().addDocumentListener(new MinMetricDocumentListener(minPhi, 8));
		
		// Create interaction explanation pane
		JLabel label = new JLabel("Interaction description", SwingConstants.CENTER);
		label.setAlignmentX(Component.CENTER_ALIGNMENT);
		label.setFont(new Font("Dialog", Font.BOLD, 15));
		JPanel interactionPane = SlickerFactory.instance().createRoundedPanel(1, new Color(160, 160, 160));
		interactionPane.add(label);
		connections.interactionPane = interactionPane;
		// TODO: move this somewhere
		
		connections.metricsPane = this;
	}
	
	private void createTable(int width) {
		int columnCount = table.getColumnModel().getColumnCount();
		singleSize = new Dimension((width - 0) / (columnCount + 4),23);
		doubleSize = new Dimension((width - 40) * 2 / (columnCount + 4),23);
		tripleSize = new Dimension((width - 120) * 3 / (columnCount + 4),23);
		switchRelationTypeBox.setMinimumSize(doubleSize);
		toggleTypeNameBox.setMinimumSize(singleSize);
		nameFilter.setMinimumSize(tripleSize);
		minSupport.setMinimumSize(singleSize);
		minConfidence.setMinimumSize(singleSize);
		minLift.setMinimumSize(singleSize);
		minConviction.setMinimumSize(singleSize);
		minCosine.setMinimumSize(singleSize);
		minJaccard.setMinimumSize(singleSize);
		minPhi.setMinimumSize(singleSize);
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.weightx = 2;
		constraints.gridwidth = 2;
		panel.add(switchRelationTypeBox, constraints);
		constraints.weightx = 1;
		constraints.gridwidth = 1;
		panel.add(toggleTypeNameBox, constraints);
		constraints.weightx = 3;
		constraints.gridwidth = 3;
		panel.add(nameFilter, constraints);
		constraints.weightx = 1;
		constraints.gridwidth = 1;
		panel.add(minSupport, constraints);
		panel.add(minConfidence, constraints);
		panel.add(minLift, constraints);
		panel.add(minConviction, constraints);
		panel.add(minCosine, constraints);
		panel.add(minJaccard, constraints);
		panel.add(minPhi, constraints);
		
		panel.setPreferredSize(new Dimension(width, 23));
		panel.setMinimumSize(new Dimension(width, 23));
		panel.setBackground(Color.LIGHT_GRAY);
		
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		// TODO: add selection logic
//		table.getSelectionModel().addListSelectionListener(new RowListener());
		table.setMinimumSize(new Dimension(200, 40));
		table.setPreferredSize(new Dimension(1000, 90));
		table.getTableHeader().setPreferredSize(new Dimension(1000,20));
		table.getTableHeader().setReorderingAllowed(false);
		
		metricsTableModel.addColumn("Condition");
		metricsTableModel.addColumn("Consequence");
		metricsTableModel.addColumn("Support");
		metricsTableModel.addColumn("Confidence");
		metricsTableModel.addColumn("Lift");
		metricsTableModel.addColumn("Conviction");
		metricsTableModel.addColumn("Cosine");
		metricsTableModel.addColumn("Jaccard");
		metricsTableModel.addColumn("Phi");
		
		Enumeration<TableColumn> columns = table.getColumnModel().getColumns();
		while (columns.hasMoreElements()) {
			final TableColumn column = columns.nextElement();
			column.setPreferredWidth(width / (columnCount + 4));
			column.setCellEditor(new DefaultCellEditor(new JTextField()) {

				protected void fireEditingStopped() {
					this.cancelCellEditing();
					super.fireEditingStopped();
				}

				protected void fireEditingCanceled() {
					super.fireEditingCanceled();
				}

			});
		}
		table.getColumnModel().getColumn(0).setPreferredWidth(3*(width / (columnCount + 4)));
		table.getColumnModel().getColumn(1).setPreferredWidth(3*(width / (columnCount + 4)));
	}
	
	DefaultTableModel metricsTableModel = new DefaultTableModel() {
		public Class getColumnClass(int column) {
			switch (column) {
				case 0: case 1:
					return String.class;
				case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9:
					return Double.class;
				default:
					return Object.class;
			}
		}
	};
}
